/// SPDX-License-Identifier: GPL-3.0-or-later
/// SPDX-FileCopyrightText: Copyright © 2016-2023 Tony Garnock-Jones <tonyg@leastfixedpoint.com>

import * as S from '../gen/sturdy.js';
import { Decoder, DecoderState, Encoder, EncoderState, GenericEmbedded, neverEmbeddedType, EmbeddedType, Value, EmbeddedWriter } from '@preserves/core';

export const wireRefEmbeddedType: EmbeddedType<S.WireRef> & EmbeddedWriter<S.WireRef> = {
    decode(s: DecoderState): S.WireRef {
        return S.asWireRef(new Decoder<any>(s).next());
    },

    encode(s: EncoderState, v: S.WireRef): void {
        new Encoder<any>(s, neverEmbeddedType).push(S.fromWireRef(v));
    },

    fromValue(v: Value<GenericEmbedded>): S.WireRef {
        return S.asWireRef(v as Value<S._embedded>);
    },

    toValue(v: S.WireRef): Value<GenericEmbedded> {
        return S.fromWireRef(v) as Value<GenericEmbedded>;
    }
};
